/*  Copyright (C) 2008 Joshua Judson Rosen <rozzin@geekspace.com>.

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 3 as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING. If not, see
    <http://www.gnu.org/licenses/> or write to:

        The Free Software Foundation, inc.
        51 Franklin Street, Fifth Floor
        Boston, MA 02110-1301
        USA
*/

#include <stdlib.h>
#include "constants.h"
#include "random.h"

#include <math.h>
#include <cairo.h>
#include "graphics.h"

#include "radial.h"
#include "spiral.h"
#include "line.h"
#include "figure.h"
#include "metashape.h"
#include "shape.h"
#include "relaxed-inside.h"
#include "path.h"
#include "symmetry.h"

static generator_fn *terminals[] = { gen_line /*, gen_scribble */ };
static const int nterminals = (sizeof terminals)/(sizeof (generator_fn *));

       generator_fn *children[] = { gen_radial,
                                    gen_spiral,
                                    gen_line,
                                    gen_figure,
                                    gen_shape,
                                    gen_relaxed_inside,
                                    gen_path,
                                    gen_symmetry };
       const int nchildren = (sizeof children)/(sizeof (generator_fn *));


static struct generator *
rnd_pick_terminal (int level)
{
  return terminals[(int) floor (rndin (0, (double) nterminals))] (level);
}

static struct generator *
rnd_pick_child (int level)
{
  return children[(int) floor (rndin (0, (double) nchildren))] (level);
}

struct generator *
generate_child (int level)
{
  if (level < recursion_max) {
    return rnd_pick_child (level);
  } else {
    return rnd_pick_terminal (level);
  }
}

void
exec_generator (struct generator *generator, cairo_t *cr)
{
  cairo_save (cr);
  generator->produce (generator, cr);
  cairo_restore (cr);
}

void
visualid_draw (struct generator *gen, cairo_t *cr,
               double width, double height,
               double linewidth)
{
  double bbx0, bbx1, bby0, bby1;
  double bbwidth, bbheight;
  double rescale;

/*   cairo_set_line_width (cr, linewidth); */

  cairo_save (cr);

  cairo_set_line_width (cr, linewidth);

  cairo_scale (cr, width/2, -height/2);
  cairo_translate (cr, 1, -1);

/*   cairo_set_line_width (cr, .03); */

  exec_generator (gen, cr);

  cairo_stroke_extents (cr, &bbx0, &bby0,
                            &bbx1, &bby1);

  bbwidth = bbx1 - bbx0;
  bbheight = bby1 - bby0;

  rescale = 2 / fmax (bbwidth, bbheight);

  cairo_new_path (cr);

  cairo_scale (cr, rescale, rescale);
  cairo_translate (cr, -(bbx0+bbx1)/2, -(bby0+bby1)/2);

  exec_generator (gen, cr);

  cairo_restore (cr);
}

/*
  Want to mutate a base generator based on:
  - how different the new string is from the base string

  What does the `different' mean?

  The `sameness' is characterised by the length of the string in common.

  string1_len / (string1_start + 1)

  or: 1 - score

  o: ...
 */
/* struct generator * */
/* mutate_generator (struct generator *base, int level, double amount) */
/* { */
/*   mutator_fn *mutate = get_mutator (base, level, amount); */
/*   struct generator *new_gen = mutate (base, level, amount); */
/*   return new_gen; */
/* } */
